import { StatusBar } from 'expo-status-bar'
import React, { useEffect } from 'react'
import { StyleSheet, View } from 'react-native'
import Animated, {
  useSharedValue,
  Easing,
  useAnimatedStyle,
  interpolate,
  withRepeat,
} from 'react-native-reanimated'
import { withKeyframes } from 'react-native-reanimated-keyframes'

const SPINNER_SIZE = 64

export default function App() {
  const animatedValue = useSharedValue(0)

  useEffect(() => {
    animatedValue.value = withRepeat(
      withKeyframes({
        duration: 1200,
        keyframes: [0, 50, 100],
        easing: Easing.bezier(0.42, 0.0, 0.58, 1.0),
      }),
      -1
    )
  }, [animatedValue])

  const animatedStyle = useAnimatedStyle(() => {
    return {
      width: SPINNER_SIZE,
      height: SPINNER_SIZE,
      backgroundColor: '#fff',
      transform: [
        {
          perspective: SPINNER_SIZE * 3,
        },
        {
          rotateX: `${interpolate(
            animatedValue.value,
            [0, 50, 100],
            [0.1, -179.9, -179.9]
          )}deg`,
        },
        {
          rotateY: `${interpolate(
            animatedValue.value,
            [0, 50, 100],
            [0.1, 0.1, -179.9]
          )}deg`,
        },
      ],
    }
  })

  return (
    <View style={styles.container}>
      <Animated.View style={animatedStyle} />
      <StatusBar style="auto" />
    </View>
  )
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: '#d35400',
    alignItems: 'center',
    justifyContent: 'center',
  },
})
