function keyframes({
  keyframes,
  easing,
  easingsByKeyframe = {}
}) {
  'worklet';

  validateKeyframes(keyframes);
  return t => {
    'worklet';

    for (let keyframeIndex = 0; keyframeIndex < keyframes.length - 1; keyframeIndex++) {
      const ratioUntilNextKeyframe = keyframes[keyframeIndex + 1] / 100;

      if (t < ratioUntilNextKeyframe) {
        var _easingsByKeyframe$ke;

        const ratioUntilCurrentKeyframe = keyframes[keyframeIndex] / 100;
        const currentKeyframeTotalRatio = (keyframes[keyframeIndex + 1] - keyframes[keyframeIndex]) / 100;
        const currentRatio = (t - ratioUntilCurrentKeyframe) / currentKeyframeTotalRatio;
        const currentEasing = (_easingsByKeyframe$ke = easingsByKeyframe[keyframes[keyframeIndex]]) !== null && _easingsByKeyframe$ke !== void 0 ? _easingsByKeyframe$ke : easing;
        return ratioUntilCurrentKeyframe + currentEasing(currentRatio) * currentKeyframeTotalRatio;
      }
    }

    return t;
  };
}

function validateKeyframes(keyframes) {
  if (keyframes.length < 3) {
    throw new Error('keyframes must have at least 3 items');
  }

  if (keyframes[0] !== 0 || keyframes[keyframes.length - 1] !== 100) {
    throw new Error("The first keyframe must be 0 and the last keyframe must be 100: ".concat(JSON.stringify(keyframes)));
  }

  for (let i = 0; i < keyframes.length; i++) {
    const currentKeyframe = keyframes[i];

    if (isNaN(currentKeyframe)) {
      throw new Error('A keyframe value must be a number');
    }

    if (i > 0) {
      const prevKeyframe = keyframes[i - 1];

      if (currentKeyframe <= prevKeyframe) {
        throw new Error("keyframes must be monotonically increasing: ".concat(JSON.stringify(keyframes)));
      }
    }
  }
}

const Easing = {
  keyframes
};
export { Easing };
//# sourceMappingURL=easing.js.map