import Animated, { withTiming } from 'react-native-reanimated'
import { Easing, KeyframesConfig } from './easing'

interface WithKeyframesConfig {
  keyframes: KeyframesConfig['keyframes']
  easing?: Animated.EasingFunction
  easingsByKeyframe?: KeyframesConfig['easingsByKeyframe']
  duration?: number
  callback?: (isFinished: boolean) => void
}

function withKeyframes({
  keyframes,
  easing,
  easingsByKeyframe,
  callback,
  duration,
}: WithKeyframesConfig) {
  return withTiming(
    100,
    {
      easing: Easing.keyframes({
        keyframes: keyframes,
        easing,
        easingsByKeyframe,
      }),
      duration,
    },
    callback
  )
}

export { withKeyframes }
